@@ -120,7 +120,8 @@ class Selection(Accessible):
                 @return True if the child was successfully selected, False otherwise.
                 """
                 func = self.get_dbus_method("SelectChild", dbus_interface=ATSPI_SELECTION)
-                return func(index)
+		val = func(index)
+		return val
 
         def get_nSelectedChildren(self):
                 return dbus.Int32(self._pgetter(ATSPI_SELECTION, "NSelectedChildren"))
